@@ -1,7 +1,10 @@
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Sequence, Type, Union
+from collections.abc import Mapping, Sequence
+from typing import TYPE_CHECKING, Any, Union
 
 from typing_extensions import TypeAlias
 
+from dagster._annotations import public
+
 if TYPE_CHECKING:
     from dagster._config import ConfigType, Field
 
@@ -13,8 +16,8 @@ if TYPE_CHECKING:
 # Unfortunately mypy doesn't support recursive types, which would be used to properly define the
 # List/Dict elements of this union: `Dict[str, ConfigSchema]`, `List[ConfigSchema]`.
 UserConfigSchema: TypeAlias = Union[
-    Type[Union[bool, float, int, str]],
-    Type[Union[Dict[Any, Any], List[Any]]],
+    type[Union[bool, float, int, str]],
+    type[Union[dict[Any, Any], list[Any]]],
     "ConfigType",
     "Field",
     Mapping[str, Any],
@@ -22,6 +25,7 @@ UserConfigSchema: TypeAlias = Union[
 ]
 
 
+@public
 class ConfigSchema:
     """Placeholder type for config schemas.
 
