@@ -1,25 +1,26 @@
-import logging
 from typing import TYPE_CHECKING, Any, Callable, Optional, Union, cast, overload
 
 import dagster._check as check
 from dagster._annotations import public
-from dagster._core.errors import DagsterInvalidInvocationError
-
-from ..decorator_utils import get_function_params
-from .config import is_callable_valid_config_arg
-from .configurable import AnonymousConfigurableDefinition
-from .definition_config_schema import (
+from dagster._core.decorator_utils import get_function_params
+from dagster._core.definitions.config import is_callable_valid_config_arg
+from dagster._core.definitions.configurable import AnonymousConfigurableDefinition
+from dagster._core.definitions.definition_config_schema import (
     CoercableToConfigSchema,
     convert_user_facing_definition_config_schema,
 )
+from dagster._core.errors import DagsterInvalidInvocationError
 
 if TYPE_CHECKING:
-    from dagster._core.definitions import JobDefinition, PipelineDefinition
+    import logging
+
+    from dagster._core.definitions import JobDefinition
     from dagster._core.execution.context.logger import InitLoggerContext, UnboundInitLoggerContext
 
     InitLoggerFunction = Callable[[InitLoggerContext], logging.Logger]
 
 
+@public
 class LoggerDefinition(AnonymousConfigurableDefinition):
     """Core class for defining loggers.
 
@@ -29,7 +30,7 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
     Args:
         logger_fn (Callable[[InitLoggerContext], logging.Logger]): User-provided function to
             instantiate the logger. This logger will be automatically invoked whenever the methods
-            on ``context.log`` are called from within job/pipeline compute logic.
+            on ``context.log`` are called from within job compute logic.
         config_schema (Optional[ConfigSchema]): The schema for the config. Configuration data available in
             `init_context.logger_config`. If not set, Dagster will accept any config provided.
         description (Optional[str]): A human-readable description of this logger.
@@ -46,10 +47,9 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
         self._description = check.opt_str_param(description, "description")
 
     def __call__(self, *args, **kwargs):
+        from dagster._core.definitions.logger_invocation import logger_invocation_result
         from dagster._core.execution.context.logger import UnboundInitLoggerContext
 
-        from .logger_invocation import logger_invocation_result
-
         if len(args) == 0 and len(kwargs) == 0:
             raise DagsterInvalidInvocationError(
                 "Logger initialization function has context argument, but no context argument was "
@@ -68,7 +68,7 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
                 args[0],
                 context_param_name,
                 UnboundInitLoggerContext,
-                default=UnboundInitLoggerContext(logger_config=None, pipeline_def=None),
+                default=UnboundInitLoggerContext(logger_config=None, job_def=None),
             )
             return logger_invocation_result(self, context)
         else:
@@ -80,7 +80,7 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
                 kwargs[context_param_name],
                 context_param_name,
                 UnboundInitLoggerContext,
-                default=UnboundInitLoggerContext(logger_config=None, pipeline_def=None),
+                default=UnboundInitLoggerContext(logger_config=None, job_def=None),
             )
 
             return logger_invocation_result(self, context)
@@ -88,16 +88,21 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
     @public
     @property
     def logger_fn(self) -> "InitLoggerFunction":
+        """Callable[[InitLoggerContext], logging.Logger]: The function that will be invoked to
+        instantiate the logger.
+        """
         return self._logger_fn
 
     @public
     @property
     def config_schema(self) -> Any:
+        """Any: The schema for the logger's config. Configuration data available in `init_context.logger_config`."""
         return self._config_schema
 
     @public
     @property
     def description(self) -> Optional[str]:
+        """Optional[str]: A human-readable description of the logger."""
         return self._description
 
     def copy_for_configured(
@@ -115,17 +120,16 @@ class LoggerDefinition(AnonymousConfigurableDefinition):
 @overload
 def logger(
     config_schema: CoercableToConfigSchema, description: Optional[str] = ...
-) -> Callable[["InitLoggerFunction"], "LoggerDefinition"]:
-    ...
+) -> Callable[["InitLoggerFunction"], "LoggerDefinition"]: ...
 
 
 @overload
 def logger(
     config_schema: "InitLoggerFunction", description: Optional[str] = ...
-) -> "LoggerDefinition":
-    ...
+) -> "LoggerDefinition": ...
 
 
+@public
 def logger(
     config_schema: Union[CoercableToConfigSchema, "InitLoggerFunction"] = None,
     description: Optional[str] = None,
@@ -156,9 +160,9 @@ def logger(
     return _wrap
 
 
+@public
 def build_init_logger_context(
     logger_config: Any = None,
-    pipeline_def: Optional["PipelineDefinition"] = None,
     job_def: Optional["JobDefinition"] = None,
 ) -> "UnboundInitLoggerContext":
     """Builds logger initialization context from provided parameters.
@@ -170,8 +174,6 @@ def build_init_logger_context(
 
     Args:
         logger_config (Any): The config to provide during initialization of logger.
-        pipeline_def (Optional[PipelineDefinition]): The pipeline definition that the logger will be
-            used with.
         job_def (Optional[JobDefinition]): The job definition that the logger will be used with.
 
     Examples:
@@ -180,20 +182,9 @@ def build_init_logger_context(
             context = build_init_logger_context()
             logger_to_init(context)
     """
-    from dagster._core.definitions import JobDefinition, PipelineDefinition
+    from dagster._core.definitions import JobDefinition
     from dagster._core.execution.context.logger import UnboundInitLoggerContext
 
-    check.opt_inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
     check.opt_inst_param(job_def, "job_def", JobDefinition)
 
-    check.invariant(
-        not (pipeline_def and job_def),
-        (
-            "In build_init_logger_context, you may only specify one of the pipeline_def and job_def"
-            " parameters, not both."
-        ),
-    )
-
-    return UnboundInitLoggerContext(
-        logger_config=logger_config, pipeline_def=pipeline_def or job_def
-    )
+    return UnboundInitLoggerContext(logger_config=logger_config, job_def=job_def)
