@@ -6,7 +6,8 @@
 # See: https://github.com/python/mypy/issues/7281
 
 from collections import namedtuple
-from typing import AbstractSet, Any, Mapping, NamedTuple, Optional
+from collections.abc import Mapping
+from typing import AbstractSet, Any, NamedTuple, Optional  # noqa: UP035
 
 import dagster._check as check
 from dagster._core.errors import DagsterUnknownResourceError
@@ -28,6 +29,10 @@ class Resources:
     def __getattr__(self, name: str) -> Any:
         raise DagsterUnknownResourceError(name)
 
+    @property
+    def original_resource_dict(self) -> Mapping[str, object]:
+        raise NotImplementedError()
+
 
 class ScopedResourcesBuilder(
     NamedTuple(
@@ -46,7 +51,7 @@ class ScopedResourcesBuilder(
         resource_instance_dict: Optional[Mapping[str, object]] = None,
         contains_generator: bool = False,
     ):
-        return super(ScopedResourcesBuilder, cls).__new__(
+        return super().__new__(
             cls,
             resource_instance_dict=check.opt_mapping_param(
                 resource_instance_dict, "resource_instance_dict", key_type=str
@@ -55,6 +60,8 @@ class ScopedResourcesBuilder(
         )
 
     def build(self, required_resource_keys: Optional[AbstractSet[str]]) -> Resources:
+        from dagster._config.pythonic_config import IAttachDifferentObjectToOpContext
+
         """We dynamically create a type that has the resource keys as properties, to enable dotting into
         the resources from a context.
 
@@ -78,6 +85,14 @@ class ScopedResourcesBuilder(
             for key in required_resource_keys
             if key in self.resource_instance_dict
         }
+        resources_to_attach_to_context = {
+            k: (
+                v.get_object_to_set_on_execution_context()
+                if isinstance(v, IAttachDifferentObjectToOpContext)
+                else v
+            )
+            for k, v in resource_instance_dict.items()
+        }
 
         # If any of the resources are generators, add the IContainsGenerator subclass to flag that
         # this is the case.
@@ -85,21 +100,31 @@ class ScopedResourcesBuilder(
 
             class _ScopedResourcesContainsGenerator(
                 namedtuple(
-                    "_ScopedResourcesContainsGenerator", list(resource_instance_dict.keys())
+                    "_ScopedResourcesContainsGenerator",
+                    list(resources_to_attach_to_context.keys()),
                 ),
                 Resources,
                 IContainsGenerator,
             ):
-                ...
+                @property
+                def original_resource_dict(self) -> Mapping[str, object]:
+                    return resource_instance_dict
 
-            return _ScopedResourcesContainsGenerator(**resource_instance_dict)  # type: ignore[call-arg]
+            return _ScopedResourcesContainsGenerator(**resources_to_attach_to_context)
 
         else:
 
             class _ScopedResources(
-                namedtuple("_ScopedResources", list(resource_instance_dict.keys())),
+                namedtuple("_ScopedResources", list(resources_to_attach_to_context.keys())),
                 Resources,
             ):
-                ...
+                @property
+                def original_resource_dict(self) -> Mapping[str, object]:
+                    return resource_instance_dict
 
-            return _ScopedResources(**resource_instance_dict)  # type: ignore[call-arg]
+            return _ScopedResources(**resources_to_attach_to_context)
+
+    @classmethod
+    def build_empty(cls) -> Resources:
+        """Returns an empty Resources object, equivalent to ScopedResourcesBuilder().build(None)."""
+        return cls().build(None)
