@@ -1,7 +1,9 @@
-from typing import Any, Mapping, Optional, Union
+import asyncio
+from collections.abc import Mapping
+from typing import Any, Optional, Union
 
 import dagster._check as check
-from dagster._annotations import public
+from dagster._annotations import deprecated, public
 from dagster._core.definitions.resource_definition import (
     IContainsGenerator,
     ResourceDefinition,
@@ -10,28 +12,15 @@ from dagster._core.definitions.resource_definition import (
 from dagster._core.errors import DagsterInvariantViolationError
 from dagster._core.instance import DagsterInstance
 from dagster._core.log_manager import DagsterLogManager
-from dagster._core.storage.pipeline_run import DagsterRun
+from dagster._core.storage.dagster_run import DagsterRun
 
 
+@public
 class InitResourceContext:
     """The context object available as the argument to the initialization function of a :py:class:`dagster.ResourceDefinition`.
 
     Users should not instantiate this object directly. To construct an `InitResourceContext` for testing purposes, use :py:func:`dagster.build_init_resource_context`.
 
-    Attributes:
-        resource_config (Any): The configuration data provided by the run config. The schema
-            for this data is defined by the ``config_field`` argument to
-            :py:class:`ResourceDefinition`.
-        resource_def (ResourceDefinition): The definition of the resource currently being
-            constructed.
-        log_manager (DagsterLogManager): The log manager for this run of the job or pipeline
-        resources (ScopedResources): The resources that are available to the resource that we are
-            initalizing.
-        dagster_run (Optional[DagsterRun]): The dagster run to use. When initializing resources
-            outside of execution context, this will be None.
-        run_id (Optional[str]): The id for this run of the job or pipeline. When initializing resources
-            outside of execution context, this will be None.
-
     Example:
         .. code-block:: python
 
@@ -46,68 +35,109 @@ class InitResourceContext:
         self,
         resource_config: Any,
         resources: Resources,
-        resource_def: Optional[ResourceDefinition] = None,
+        resource_def: Optional[ResourceDefinition],
+        all_resource_defs: Mapping[str, ResourceDefinition],
         instance: Optional[DagsterInstance] = None,
         dagster_run: Optional[DagsterRun] = None,
         log_manager: Optional[DagsterLogManager] = None,
+        event_loop: Optional[asyncio.AbstractEventLoop] = None,
     ):
         self._resource_config = resource_config
         self._resource_def = resource_def
+        self._all_resource_defs = all_resource_defs
         self._log_manager = log_manager
         self._instance = instance
         self._resources = resources
         self._dagster_run = dagster_run
+        self._event_loop = event_loop
 
     @public
     @property
     def resource_config(self) -> Any:
+        """The configuration data provided by the run config. The schema
+        for this data is defined by the ``config_field`` argument to
+        :py:class:`ResourceDefinition`.
+        """
         return self._resource_config
 
     @public
     @property
-    def resource_def(self) -> Optional[ResourceDefinition]:
-        return self._resource_def
+    def resource_def(self) -> ResourceDefinition:
+        """The definition of the resource currently being constructed."""
+        return check.not_none(self._resource_def)
 
     @public
     @property
     def resources(self) -> Resources:
+        """The resources that are available to the resource that we are initializing."""
         return self._resources
 
     @public
     @property
     def instance(self) -> Optional[DagsterInstance]:
+        """The Dagster instance configured for the current execution context."""
         return self._instance
 
+    @public
+    @property
+    def run(self) -> Optional[DagsterRun]:
+        """The dagster run to use. When initializing resources outside of execution context, this will be None."""
+        return self._dagster_run
+
+    @deprecated(
+        breaking_version="a future release",
+        subject="InitResourceContext.dagster_run",
+        additional_warn_text="You have called the deprecated method dagster_run on InitResourceContext. Use context.run instead.",
+    )
     @property
     def dagster_run(self) -> Optional[DagsterRun]:
+        """The dagster run to use. When initializing resources outside of execution context, this will be None."""
         return self._dagster_run
 
     @public
     @property
     def log(self) -> Optional[DagsterLogManager]:
+        """The Dagster log manager configured for the current execution context."""
         return self._log_manager
 
     # backcompat: keep around this property from when InitResourceContext used to be a NamedTuple
     @public
     @property
     def log_manager(self) -> Optional[DagsterLogManager]:
+        """The log manager for this run of the job."""
         return self._log_manager
 
-    @public
+    @deprecated(
+        breaking_version="a future release",
+        subject="InitResourceContext.run_id",
+        additional_warn_text="You have called the deprecated method run_id on InitResourceContext. Use context.run.run_id instead.",
+    )
     @property
     def run_id(self) -> Optional[str]:
+        """The id for this run of the job or pipeline. When initializing resources outside of
+        execution context, this will be None.
+        """
         return self.dagster_run.run_id if self.dagster_run else None
 
+    @property
+    def all_resource_defs(self) -> Mapping[str, ResourceDefinition]:
+        return self._all_resource_defs
+
     def replace_config(self, config: Any) -> "InitResourceContext":
         return InitResourceContext(
             resource_config=config,
             resources=self.resources,
             instance=self.instance,
             resource_def=self.resource_def,
+            all_resource_defs=self.all_resource_defs,
             dagster_run=self.dagster_run,
             log_manager=self.log,
         )
 
+    @property
+    def event_loop(self) -> Optional[asyncio.AbstractEventLoop]:
+        return self._event_loop
+
 
 class UnboundInitResourceContext(InitResourceContext):
     """Resource initialization context outputted by ``build_init_resource_context``.
@@ -130,7 +160,7 @@ class UnboundInitResourceContext(InitResourceContext):
             build_resources,
             wrap_resources_for_execution,
         )
-        from dagster._core.execution.context_creation_pipeline import initialize_console_manager
+        from dagster._core.execution.context_creation_job import initialize_console_manager
 
         self._instance_provided = (
             check.opt_inst_param(instance, "instance", DagsterInstance) is not None
@@ -143,6 +173,7 @@ class UnboundInitResourceContext(InitResourceContext):
 
         if isinstance(resources, Resources):
             check.failed("Should not have a Resources object directly from this initialization")
+        self._raw_resources = resources
 
         self._resource_defs = wrap_resources_for_execution(
             check.opt_mapping_param(resources, "resources")
@@ -153,10 +184,11 @@ class UnboundInitResourceContext(InitResourceContext):
         self._resources_contain_cm = isinstance(resources, IContainsGenerator)
 
         self._cm_scope_entered = False
-        super(UnboundInitResourceContext, self).__init__(
+        super().__init__(
             resource_config=resource_config,
             resources=resources,
             resource_def=None,
+            all_resource_defs={},
             instance=instance,
             dagster_run=None,
             log_manager=initialize_console_manager(None),
@@ -182,13 +214,14 @@ class UnboundInitResourceContext(InitResourceContext):
         return self._resource_config
 
     @property
-    def resource_def(self) -> Optional[ResourceDefinition]:
+    def resource_def(self) -> Optional[ResourceDefinition]:  # pyright: ignore[reportIncompatibleMethodOverride]
         raise DagsterInvariantViolationError(
-            "UnboundInitLoggerContext has not been validated against a logger definition."
+            "UnboundInitResourceContext has not been bound to resource definition."
         )
 
     @property
     def resources(self) -> Resources:
+        """The resources that are available to the resource that we are initalizing."""
         if self._resources_cm and self._resources_contain_cm and not self._cm_scope_entered:
             raise DagsterInvariantViolationError(
                 "At least one provided resource is a generator, but attempting to access "
@@ -214,7 +247,15 @@ class UnboundInitResourceContext(InitResourceContext):
     def run_id(self) -> Optional[str]:
         return None
 
+    def replace_config(self, config: Any) -> "UnboundInitResourceContext":
+        return UnboundInitResourceContext(
+            resource_config=config,
+            resources=self._raw_resources,
+            instance=self.instance,
+        )
 
+
+@public
 def build_init_resource_context(
     config: Optional[Mapping[str, Any]] = None,
     resources: Optional[Mapping[str, Any]] = None,
