@@ -1,5 +1,6 @@
 from abc import ABC, abstractmethod
-from typing import Iterator, Mapping, NamedTuple, Optional, Sequence
+from collections.abc import Iterator, Mapping, Sequence
+from typing import NamedTuple, Optional
 
 from dagster import (
     DagsterInstance,
@@ -8,7 +9,7 @@ from dagster import (
 from dagster._core.events import DagsterEvent
 from dagster._core.execution.context.system import IStepContext, PlanOrchestrationContext
 from dagster._core.execution.plan.step import ExecutionStep
-from dagster._core.storage.pipeline_run import DagsterRun
+from dagster._core.storage.dagster_run import DagsterRun
 from dagster._grpc.types import ExecuteStepArgs
 
 
@@ -22,10 +23,10 @@ class StepHandlerContext:
         dagster_run: Optional[DagsterRun] = None,
     ) -> None:
         self._instance = instance
-        self._dagster_run = plan_context
+        self._plan_context = plan_context
         self._steps_by_key = {step.key: step for step in steps}
         self._execute_step_args = execute_step_args
-        self._pipeline_run = dagster_run
+        self._dagster_run = dagster_run
 
     @property
     def execute_step_args(self) -> ExecuteStepArgs:
@@ -34,14 +35,14 @@ class StepHandlerContext:
     @property
     def dagster_run(self) -> DagsterRun:
         # lazy load
-        if not self._pipeline_run:
-            run_id = self.execute_step_args.pipeline_run_id
+        if not self._dagster_run:
+            run_id = self.execute_step_args.run_id
             run = self._instance.get_run_by_id(run_id)
             if run is None:
                 check.failed(f"Failed to load run {run_id}")
-            self._pipeline_run = run
+            self._dagster_run = run
 
-        return self._pipeline_run
+        return self._dagster_run
 
     @property
     def step_tags(self) -> Mapping[str, Mapping[str, str]]:
@@ -52,7 +53,7 @@ class StepHandlerContext:
         return self._instance
 
     def get_step_context(self, step_key: str) -> IStepContext:
-        return self._dagster_run.for_step(self._steps_by_key[step_key])
+        return self._plan_context.for_step(self._steps_by_key[step_key])
 
 
 class CheckStepHealthResult(
@@ -61,7 +62,7 @@ class CheckStepHealthResult(
     )
 ):
     def __new__(cls, is_healthy: bool, unhealthy_reason: Optional[str] = None):
-        return super(CheckStepHealthResult, cls).__new__(
+        return super().__new__(
             cls,
             check.bool_param(is_healthy, "is_healthy"),
             check.opt_str_param(unhealthy_reason, "unhealthy_reason"),
