@@ -1,35 +1,38 @@
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import NamedTuple, Optional
+from typing import TYPE_CHECKING, NamedTuple, Optional
 
+from dagster._annotations import public
 from dagster._core.instance import MayHaveInstanceWeakref, T_DagsterInstance
-from dagster._core.origin import PipelinePythonOrigin
-from dagster._core.storage.pipeline_run import DagsterRun
-from dagster._core.workspace.workspace import IWorkspace
+from dagster._core.origin import JobPythonOrigin
+from dagster._core.storage.dagster_run import DagsterRun
 from dagster._serdes import whitelist_for_serdes
 
+if TYPE_CHECKING:
+    from dagster._core.workspace.context import BaseWorkspaceRequestContext
+
 
 class LaunchRunContext(NamedTuple):
     """Context available within a run launcher's launch_run call."""
 
     dagster_run: DagsterRun
-    workspace: Optional[IWorkspace]
+    workspace: Optional["BaseWorkspaceRequestContext"]
 
     @property
-    def pipeline_code_origin(self) -> Optional[PipelinePythonOrigin]:
-        return self.dagster_run.pipeline_code_origin
+    def job_code_origin(self) -> Optional[JobPythonOrigin]:
+        return self.dagster_run.job_code_origin
 
 
 class ResumeRunContext(NamedTuple):
     """Context available within a run launcher's resume_run call."""
 
     dagster_run: DagsterRun
-    workspace: Optional[IWorkspace]
+    workspace: Optional["BaseWorkspaceRequestContext"]
     resume_attempt_number: Optional[int] = None
 
     @property
-    def pipeline_code_origin(self) -> Optional[PipelinePythonOrigin]:
-        return self.dagster_run.pipeline_code_origin
+    def job_code_origin(self) -> Optional[JobPythonOrigin]:
+        return self.dagster_run.job_code_origin
 
 
 @whitelist_for_serdes
@@ -46,11 +49,14 @@ class CheckRunHealthResult(NamedTuple):
 
     status: WorkerStatus
     msg: Optional[str] = None
+    transient: Optional[bool] = None
+    run_worker_id: Optional[str] = None  # Identifier for a particular run worker
 
     def __str__(self) -> str:
         return f"{self.status.value}: '{self.msg}'"
 
 
+@public
 class RunLauncher(ABC, MayHaveInstanceWeakref[T_DagsterInstance]):
     @abstractmethod
     def launch_run(self, context: LaunchRunContext) -> None:
@@ -65,7 +71,7 @@ class RunLauncher(ABC, MayHaveInstanceWeakref[T_DagsterInstance]):
         Args:
             context (LaunchRunContext): information about the launch - every run launcher
             will need the PipelineRun, and some run launchers may need information from the
-            IWorkspace from which the run was launched.
+            BaseWorkspaceRequestContext from which the run was launched.
         """
 
     @abstractmethod
@@ -94,7 +100,9 @@ class RunLauncher(ABC, MayHaveInstanceWeakref[T_DagsterInstance]):
             "This run launcher does not support run monitoring. Please disable it on your instance."
         )
 
-    def get_run_worker_debug_info(self, run: DagsterRun) -> Optional[str]:
+    def get_run_worker_debug_info(
+        self, run: DagsterRun, include_container_logs: Optional[bool] = True
+    ) -> Optional[str]:
         return None
 
     @property
