@@ -1,8 +1,8 @@
-from .subset_selector import (
+from dagster._core.selector.subset_selector import (
     Traverser as Traverser,
     generate_dep_graph as generate_dep_graph,
     parse_clause as parse_clause,
     parse_items_from_selection as parse_items_from_selection,
-    parse_solid_selection as parse_solid_selection,
+    parse_op_queries as parse_op_queries,
     parse_step_selection as parse_step_selection,
 )
