@@ -5,6 +5,7 @@ Revises: da7cd32b690d
 Create Date: 2020-02-10 12:52:49.540462
 
 """
+
 from alembic import op
 from sqlalchemy import inspect
 
@@ -21,6 +22,9 @@ def upgrade():
     inspector = inspect(op.get_bind())
     has_tables = inspector.get_table_names()
 
+    if op.get_context().dialect.name == "sqlite":
+        return
+
     if "runs" in has_tables and "run_tags" in has_tables:
         op.drop_constraint("run_tags_run_id_fkey", table_name="run_tags", type_="foreignkey")
         op.create_foreign_key(
@@ -37,6 +41,9 @@ def downgrade():
     inspector = inspect(op.get_bind())
     has_tables = inspector.get_table_names()
 
+    if op.get_context().dialect.name == "sqlite":
+        return
+
     if "runs" in has_tables and "run_tags" in has_tables:
         op.drop_constraint("run_tags_run_id_fkey", table_name="run_tags", type_="foreignkey")
         op.create_foreign_key(
