@@ -1,10 +1,11 @@
-"""0.10.0 create new run tables
+"""0.10.0 create new run tables.
 
 Revision ID: 5ae53cd67d0c
 Revises: 140198fdfe65
 Create Date: 2021-01-13 14:42:51.215325
 
 """
+
 from dagster._core.storage.migration.utils import create_0_10_0_run_tables
 
 # revision identifiers, used by Alembic.
