@@ -1,12 +1,13 @@
 from typing import NamedTuple
 
 import sqlalchemy as db
+from dagster_shared.serdes import deserialize_value
 from tqdm import tqdm
 
 from dagster._core.assets import AssetDetails
 from dagster._core.events.log import EventLogEntry
-from dagster._serdes.serdes import deserialize_value
-from dagster._utils import utc_datetime_from_timestamp
+from dagster._core.storage.sqlalchemy_compat import db_select
+from dagster._time import datetime_from_timestamp
 
 SECONDARY_INDEX_ASSET_KEY = "asset_key_table"  # builds the asset key table from the event log
 ASSET_KEY_INDEX_COLS = "asset_key_index_columns"  # extracts index columns from the asset_keys table
@@ -25,12 +26,12 @@ def migrate_event_log_data(instance=None):
     """
     from dagster._core.storage.event_log.sql_event_log import SqlEventLogStorage
 
-    event_log_storage = instance._event_storage  # noqa: SLF001
+    event_log_storage = instance._event_storage  # noqa: SLF001  # pyright: ignore[reportOptionalMemberAccess]
 
     if not isinstance(event_log_storage, SqlEventLogStorage):
         return
 
-    for run in instance.get_runs():
+    for run in instance.get_runs():  # pyright: ignore[reportOptionalMemberAccess]
         for record in event_log_storage.get_records_for_run(run.run_id).records:
             event_log_storage.update_event_log_record(record.storage_id, record.event_log_entry)
 
@@ -40,15 +41,14 @@ def migrate_asset_key_data(event_log_storage, print_fn=None):
     Takes in event_log_storage, and a print_fn to keep track of progress.
     """
     from dagster._core.definitions.events import AssetKey
+    from dagster._core.storage.event_log.schema import AssetKeyTable, SqlEventLogStorageTable
     from dagster._core.storage.event_log.sql_event_log import SqlEventLogStorage
 
-    from .schema import AssetKeyTable, SqlEventLogStorageTable
-
     if not isinstance(event_log_storage, SqlEventLogStorage):
         return
 
     query = (
-        db.select([SqlEventLogStorageTable.c.asset_key])
+        db_select([SqlEventLogStorageTable.c.asset_key])
         .where(SqlEventLogStorageTable.c.asset_key != None)  # noqa: E711
         .group_by(SqlEventLogStorageTable.c.asset_key)
     )
@@ -64,21 +64,20 @@ def migrate_asset_key_data(event_log_storage, print_fn=None):
             try:
                 conn.execute(
                     AssetKeyTable.insert().values(
-                        asset_key=AssetKey.from_db_string(asset_key).to_string()
+                        asset_key=AssetKey.from_db_string(asset_key).to_string()  # pyright: ignore[reportOptionalMemberAccess]
                     )
                 )
-            except db.exc.IntegrityError:
+            except db.exc.IntegrityError:  # pyright: ignore[reportAttributeAccessIssue]
                 # asset key already present
                 pass
 
 
 def migrate_asset_keys_index_columns(event_log_storage, print_fn=None):
     from dagster._core.definitions.events import AssetKey
+    from dagster._core.storage.event_log.schema import AssetKeyTable, SqlEventLogStorageTable
     from dagster._core.storage.event_log.sql_event_log import SqlEventLogStorage
     from dagster._serdes import serialize_value
 
-    from .schema import AssetKeyTable, SqlEventLogStorageTable
-
     if not isinstance(event_log_storage, SqlEventLogStorage):
         return
 
@@ -86,7 +85,7 @@ def migrate_asset_keys_index_columns(event_log_storage, print_fn=None):
         if print_fn:
             print_fn("Querying asset keys.")
         results = conn.execute(
-            db.select(
+            db_select(
                 [
                     AssetKeyTable.c.asset_key,
                     AssetKeyTable.c.asset_details,
@@ -118,16 +117,16 @@ def migrate_asset_keys_index_columns(event_log_storage, print_fn=None):
 
             if not event:
                 materialization_query = (
-                    db.select([SqlEventLogStorageTable.c.event])
+                    db_select([SqlEventLogStorageTable.c.event])
                     .where(
-                        SqlEventLogStorageTable.c.asset_key == asset_key.to_string(),
+                        SqlEventLogStorageTable.c.asset_key == asset_key.to_string(),  # pyright: ignore[reportOptionalMemberAccess]
                     )
                     .order_by(SqlEventLogStorageTable.c.timestamp.desc())
                     .limit(1)
                 )
                 materialization_row = conn.execute(materialization_query).fetchone()
                 if materialization_row:
-                    event = deserialize_value(materialization_row[0], NamedTuple)
+                    event = deserialize_value(materialization_row[0], NamedTuple)  # pyright: ignore[reportCallIssue,reportArgumentType]
 
             if not event:
                 # this must be a wiped asset
@@ -136,12 +135,12 @@ def migrate_asset_keys_index_columns(event_log_storage, print_fn=None):
                     .values(
                         last_materialization=None,
                         last_materialization_timestamp=None,
-                        wipe_timestamp=utc_datetime_from_timestamp(wipe_timestamp)
-                        if wipe_timestamp
-                        else None,
+                        wipe_timestamp=(
+                            datetime_from_timestamp(wipe_timestamp) if wipe_timestamp else None
+                        ),
                     )
                     .where(
-                        AssetKeyTable.c.asset_key == asset_key.to_string(),
+                        AssetKeyTable.c.asset_key == asset_key.to_string(),  # pyright: ignore[reportOptionalMemberAccess]
                     )
                 )
             else:
@@ -149,22 +148,22 @@ def migrate_asset_keys_index_columns(event_log_storage, print_fn=None):
                     AssetKeyTable.update()
                     .values(
                         last_materialization=serialize_value(event),
-                        last_materialization_timestamp=utc_datetime_from_timestamp(event.timestamp),
-                        wipe_timestamp=utc_datetime_from_timestamp(wipe_timestamp)
-                        if wipe_timestamp
-                        else None,
+                        last_materialization_timestamp=datetime_from_timestamp(event.timestamp),
+                        wipe_timestamp=(
+                            datetime_from_timestamp(wipe_timestamp) if wipe_timestamp else None
+                        ),
                     )
                     .where(
-                        AssetKeyTable.c.asset_key == asset_key.to_string(),
+                        AssetKeyTable.c.asset_key == asset_key.to_string(),  # pyright: ignore[reportOptionalMemberAccess]
                     )
                 )
 
 
 def sql_asset_event_generator(conn, cursor=None, batch_size=1000):
-    from .schema import SqlEventLogStorageTable
+    from dagster._core.storage.event_log.schema import SqlEventLogStorageTable
 
     while True:
-        query = db.select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event]).where(
+        query = db_select([SqlEventLogStorageTable.c.id, SqlEventLogStorageTable.c.event]).where(
             SqlEventLogStorageTable.c.asset_key != None  # noqa: E711
         )
         if cursor:
