@@ -1,13 +1,16 @@
+import os
 import threading
-from typing import Callable, List, MutableMapping, NamedTuple, Optional
+from typing import TYPE_CHECKING, Callable, NamedTuple, Optional
 
 import dagster._check as check
 from dagster._core.events.log import EventLogEntry
-from dagster._core.storage.event_log.base import EventLogCursor
+from dagster._core.storage.event_log.base import EventLogCursor, EventLogStorage
 
-from .sql_event_log import SqlEventLogStorage
+if TYPE_CHECKING:
+    from collections.abc import MutableMapping
 
-POLLING_CADENCE = 0.1  # 100 ms
+INIT_POLL_PERIOD = 0.250  # 250ms
+MAX_POLL_PERIOD = 16.0  # 16s
 
 
 class CallbackAfterCursor(NamedTuple):
@@ -32,9 +35,9 @@ class SqlPollingEventWatcher:
         INVARIANTS: _dict_lock protects _run_id_to_watcher_dict
     """
 
-    def __init__(self, event_log_storage: SqlEventLogStorage):
+    def __init__(self, event_log_storage: EventLogStorage):
         self._event_log_storage = check.inst_param(
-            event_log_storage, "event_log_storage", SqlEventLogStorage
+            event_log_storage, "event_log_storage", EventLogStorage
         )
 
         # INVARIANT: dict_lock protects _run_id_to_watcher_dict
@@ -49,11 +52,16 @@ class SqlPollingEventWatcher:
         return _has_run_id
 
     def watch_run(
-        self, run_id: str, cursor: Optional[str], callback: Callable[[EventLogEntry, str], None]
-    ):
+        self,
+        run_id: str,
+        cursor: Optional[str],
+        callback: Callable[[EventLogEntry, str], None],
+    ) -> None:
         run_id = check.str_param(run_id, "run_id")
         cursor = check.opt_str_param(cursor, "cursor")
         callback = check.callable_param(callback, "callback")
+        check.invariant(not self._disposed, "Attempted to watch_run after close")
+
         with self._dict_lock:
             if run_id not in self._run_id_to_watcher_dict:
                 self._run_id_to_watcher_dict[run_id] = SqlPollingRunIdEventWatcherThread(
@@ -63,7 +71,11 @@ class SqlPollingEventWatcher:
                 self._run_id_to_watcher_dict[run_id].start()
             self._run_id_to_watcher_dict[run_id].add_callback(cursor, callback)
 
-    def unwatch_run(self, run_id: str, handler: Callable[[EventLogEntry, str], None]):
+    def unwatch_run(
+        self,
+        run_id: str,
+        handler: Callable[[EventLogEntry, str], None],
+    ) -> None:
         run_id = check.str_param(run_id, "run_id")
         handler = check.callable_param(handler, "handler")
         with self._dict_lock:
@@ -72,10 +84,7 @@ class SqlPollingEventWatcher:
                 if self._run_id_to_watcher_dict[run_id].should_thread_exit.is_set():
                     del self._run_id_to_watcher_dict[run_id]
 
-    def __del__(self):
-        self.close()
-
-    def close(self):
+    def close(self) -> None:
         if not self._disposed:
             self._disposed = True
             with self._dict_lock:
@@ -84,7 +93,7 @@ class SqlPollingEventWatcher:
                         watcher_thread.should_thread_exit.set()
                 for run_id in self._run_id_to_watcher_dict:
                     self._run_id_to_watcher_dict[run_id].join()
-                del self._run_id_to_watcher_dict
+                self._run_id_to_watcher_dict = {}
 
 
 class SqlPollingRunIdEventWatcherThread(threading.Thread):
@@ -100,16 +109,16 @@ class SqlPollingRunIdEventWatcherThread(threading.Thread):
 
     """
 
-    def __init__(self, event_log_storage: SqlEventLogStorage, run_id: str):
-        super(SqlPollingRunIdEventWatcherThread, self).__init__()
+    def __init__(self, event_log_storage: EventLogStorage, run_id: str):
+        super().__init__()
         self._event_log_storage = check.inst_param(
-            event_log_storage, "event_log_storage", SqlEventLogStorage
+            event_log_storage, "event_log_storage", EventLogStorage
         )
         self._run_id = check.str_param(run_id, "run_id")
         self._callback_fn_list_lock: threading.Lock = threading.Lock()
-        self._callback_fn_list: List[CallbackAfterCursor] = []
+        self._callback_fn_list: list[CallbackAfterCursor] = []
         self._should_thread_exit = threading.Event()
-        self.name = f"mysql-event-watch-run-id-{self._run_id}"
+        self.name = f"sql-event-watch-run-id-{self._run_id}"
 
     @property
     def should_thread_exit(self) -> threading.Event:
@@ -147,7 +156,7 @@ class SqlPollingRunIdEventWatcherThread(threading.Thread):
             if not self._callback_fn_list:
                 self._should_thread_exit.set()
 
-    def run(self):
+    def run(self) -> None:
         """Polling function to update Observers with EventLogEntrys from Event Log DB.
         Wakes every POLLING_CADENCE &
             1. executes a SELECT query to get new EventLogEntrys
@@ -155,9 +164,17 @@ class SqlPollingRunIdEventWatcherThread(threading.Thread):
         Uses max_index_so_far as a cursor in the DB to make sure that only new records are retrieved.
         """
         cursor = None
-        while not self._should_thread_exit.wait(POLLING_CADENCE):
-            conn = self._event_log_storage.get_records_for_run(self._run_id, cursor=cursor)
-            cursor = conn.cursor if conn.cursor else cursor
+        wait_time = INIT_POLL_PERIOD
+
+        chunk_limit = int(os.getenv("DAGSTER_POLLING_EVENT_WATCHER_BATCH_SIZE", "1000"))
+
+        while not self._should_thread_exit.wait(wait_time):
+            conn = self._event_log_storage.get_records_for_run(
+                self._run_id,
+                cursor=cursor,
+                limit=chunk_limit,
+            )
+            cursor = conn.cursor
             for event_record in conn.records:
                 with self._callback_fn_list_lock:
                     for callback_with_cursor in self._callback_fn_list:
@@ -170,3 +187,4 @@ class SqlPollingRunIdEventWatcherThread(threading.Thread):
                                 event_record.event_log_entry,
                                 str(EventLogCursor.from_storage_id(event_record.storage_id)),
                             )
+            wait_time = INIT_POLL_PERIOD if conn.records else min(wait_time * 2, MAX_POLL_PERIOD)
