@@ -1,5 +1,9 @@
 import typing
 
+from dagster._core.types.dagster_type import List, Optional
+from dagster._core.types.python_dict import Dict, create_typed_runtime_dict
+from dagster._core.types.python_set import Set
+from dagster._core.types.python_tuple import Tuple, create_typed_tuple
 from dagster._utils.typing_api import (
     get_dict_key_value_types,
     get_list_inner_type,
@@ -13,14 +17,8 @@ from dagster._utils.typing_api import (
     is_closed_python_tuple_type,
 )
 
-from .dagster_type import List, Optional
-from .python_dict import Dict, create_typed_runtime_dict
-from .python_set import Set
-from .python_tuple import Tuple, create_typed_tuple
-
 
 def transform_typing_type(type_annotation):
-
     if type_annotation is typing.List:
         return List
     elif type_annotation is typing.Set:
