@@ -1,5 +1,6 @@
 import os
-from typing import Dict, Mapping, cast
+from collections.abc import Mapping
+from typing import cast
 
 import dagster._check as check
 from dagster._config import (
@@ -11,12 +12,13 @@ from dagster._config import (
     StringSource,
     process_config,
 )
+from dagster._config.field_utils import Permissive
 from dagster._core.errors import DagsterInvalidConfigError
-from dagster._utils import merge_dicts
+from dagster._utils.merger import merge_dicts
 
 
 def process_workspace_config(
-    workspace_config: Mapping[str, object]
+    workspace_config: Mapping[str, object],
 ) -> EvaluateValueResult[Mapping[str, object]]:
     workspace_config = check.mapping_param(workspace_config, "workspace_config")
 
@@ -32,13 +34,11 @@ def ensure_workspace_config(
     validation_result = process_workspace_config(workspace_config)
     if not validation_result.success:
         raise DagsterInvalidConfigError(
-            "Errors while loading workspace config from {yaml_path}.".format(
-                yaml_path=os.path.abspath(yaml_path)
-            ),
+            f"Errors while loading workspace config from {os.path.abspath(yaml_path)}.",
             validation_result.errors,
             workspace_config,
         )
-    return cast(Dict[str, object], validation_result.value)
+    return cast("dict[str, object]", validation_result.value)
 
 
 def _get_target_config() -> Mapping[str, ScalarUnion]:
@@ -73,6 +73,15 @@ def _get_target_config() -> Mapping[str, ScalarUnion]:
                 "executable_path": Field(StringSource, is_required=False),
             },
         ),
+        "autoload_defs_module": ScalarUnion(
+            scalar_type=str,
+            non_scalar_schema={
+                "module_name": StringSource,
+                "working_directory": Field(StringSource, is_required=False),
+                "location_name": Field(StringSource, is_required=False),
+                "executable_path": Field(StringSource, is_required=False),
+            },
+        ),
     }
 
 
@@ -89,6 +98,7 @@ WORKSPACE_CONFIG_SCHEMA = {
                             "port": Field(IntSource, is_required=False),
                             "location_name": Field(StringSource, is_required=False),
                             "ssl": Field(bool, is_required=False),
+                            "additional_metadata": Field(Permissive(), is_required=False),
                         },
                     },
                 )
