@@ -51,8 +51,7 @@ from dagster._core.execution.submit_asset_runs import (
 )
 from dagster._core.instance import DagsterInstance
 from dagster._core.remote_origin import RemoteInstigatorOrigin
-from dagster._core.remote_representation import RemoteSensor
-from dagster._core.remote_representation.external import RemoteRepository
+from dagster._core.remote_representation.external import RemoteRepository, RemoteSensor
 from dagster._core.scheduler.instigation import (
     InstigatorState,
     InstigatorStatus,
@@ -466,16 +465,35 @@ class AssetDaemon(DagsterDaemon):
         if get_auto_materialize_paused(instance) and not use_auto_materialize_sensors:
             return
 
-        now = get_current_timestamp()
+        with workspace_process_context.create_request_context() as workspace_request_context:
+            self._run_iteration_impl_with_request_context(
+                workspace_process_context,
+                workspace_request_context,
+                instance,
+                threadpool_executor,
+                amp_tick_futures,
+                use_auto_materialize_sensors,
+                debug_crash_flags,
+            )
 
-        workspace = workspace_process_context.create_request_context()
+    def _run_iteration_impl_with_request_context(
+        self,
+        workspace_process_context: IWorkspaceProcessContext,
+        workspace_request_context: BaseWorkspaceRequestContext,
+        instance: DagsterInstance,
+        threadpool_executor: Optional[ThreadPoolExecutor],
+        amp_tick_futures: dict[Optional[str], Future],
+        use_auto_materialize_sensors: bool,
+        debug_crash_flags: SingleInstigatorDebugCrashFlags,
+    ):
+        now = get_current_timestamp()
 
         sensors_and_repos: Sequence[tuple[Optional[RemoteSensor], Optional[RemoteRepository]]] = []
 
         if use_auto_materialize_sensors:
             current_workspace = {
                 location_entry.origin.location_name: location_entry
-                for location_entry in workspace.get_code_location_entries().values()
+                for location_entry in workspace_request_context.get_code_location_entries().values()
             }
 
             eligible_sensors_and_repos = []
@@ -501,7 +519,7 @@ class AssetDaemon(DagsterDaemon):
                 if not get_has_migrated_to_sensors(instance):
                     # Do a one-time migration to create the cursors for each sensor, based on the
                     # existing cursor for the legacy AMP tick
-                    asset_graph = workspace.asset_graph
+                    asset_graph = workspace_request_context.asset_graph
                     pre_sensor_cursor = _get_pre_sensor_auto_materialize_cursor(
                         instance, asset_graph
                     )
@@ -596,7 +614,7 @@ class AssetDaemon(DagsterDaemon):
                 future = threadpool_executor.submit(
                     self._process_auto_materialize_tick,
                     workspace_process_context,
-                    workspace,
+                    workspace_request_context,
                     repo,
                     sensor,
                     debug_crash_flags,
@@ -605,7 +623,7 @@ class AssetDaemon(DagsterDaemon):
             else:
                 self._process_auto_materialize_tick(
                     workspace_process_context,
-                    workspace,
+                    workspace_request_context,
                     repo,
                     sensor,
                     debug_crash_flags,
@@ -902,13 +920,16 @@ class AssetDaemon(DagsterDaemon):
                     )
                 )
 
-            with AutoMaterializeLaunchContext(
-                tick,
-                sensor,
-                instance,
-                self._logger,
-                tick_retention_settings,
-            ) as tick_context:
+            with (
+                AutoMaterializeLaunchContext(
+                    tick,
+                    sensor,
+                    instance,
+                    self._logger,
+                    tick_retention_settings,
+                ) as tick_context,
+                workspace,
+            ):
                 await self._evaluate_auto_materialize_tick(
                     tick_context,
                     tick,
@@ -1163,6 +1184,7 @@ class AssetDaemon(DagsterDaemon):
                         },
                         title=f"Run for Declarative Automation evaluation ID {evaluation_id}",
                         description=None,
+                        run_config=run_request.run_config,
                     )
                 )
             return reserved_run_id, check.not_none(asset_graph_subset.asset_keys)
