@@ -1,9 +1,12 @@
+import json
+import os
 import sys
 import threading
 import time
+from collections.abc import Iterable, Iterator, Sequence
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from contextlib import ExitStack
-from typing import Dict, Iterable, Iterator, List, Optional, Sequence
+from typing import Optional
 
 from dagster import (
     DagsterEvent,
@@ -13,42 +16,47 @@ from dagster import (
 from dagster._core.errors import DagsterCodeLocationLoadError, DagsterUserCodeUnreachableError
 from dagster._core.events import EngineEventData
 from dagster._core.instance import DagsterInstance
+from dagster._core.instance.config import ConcurrencyConfig
 from dagster._core.launcher import LaunchRunContext
-from dagster._core.run_coordinator.queued_run_coordinator import (
-    QueuedRunCoordinator,
-    RunQueueConfig,
-)
-from dagster._core.storage.pipeline_run import (
+from dagster._core.op_concurrency_limits_counter import GlobalOpConcurrencyLimitsCounter
+from dagster._core.run_coordinator.queued_run_coordinator import QueuedRunCoordinator
+from dagster._core.storage.dagster_run import (
     IN_PROGRESS_RUN_STATUSES,
     DagsterRun,
     DagsterRunStatus,
+    RunRecord,
     RunsFilter,
 )
 from dagster._core.storage.tags import PRIORITY_TAG
-from dagster._core.workspace.context import IWorkspaceProcessContext
-from dagster._core.workspace.workspace import IWorkspace
+from dagster._core.utils import InheritContextThreadPoolExecutor
+from dagster._core.workspace.context import BaseWorkspaceRequestContext, IWorkspaceProcessContext
 from dagster._daemon.daemon import DaemonIterator, IntervalDaemon
-from dagster._utils.error import serializable_error_info_from_exc_info
+from dagster._daemon.utils import DaemonErrorCapture
 from dagster._utils.tags import TagConcurrencyLimitsCounter
 
+PAGE_SIZE = int(os.getenv("DAGSTER_RUN_QUEUE_PAGE_SIZE", "100"))
+
 
 class QueuedRunCoordinatorDaemon(IntervalDaemon):
     """Used with the QueuedRunCoordinator on the instance. This process finds queued runs from the run
     store and launches them.
     """
 
-    def __init__(self, interval_seconds) -> None:
+    def __init__(self, interval_seconds, page_size=PAGE_SIZE) -> None:
         self._exit_stack = ExitStack()
         self._executor: Optional[ThreadPoolExecutor] = None
         self._location_timeouts_lock = threading.Lock()
-        self._location_timeouts: Dict[str, float] = {}
+        self._location_timeouts: dict[str, float] = {}
+        self._page_size = page_size
+        self._global_concurrency_blocked_runs_lock = threading.Lock()
+        self._global_concurrency_blocked_runs = set()
         super().__init__(interval_seconds)
 
     def _get_executor(self, max_workers) -> ThreadPoolExecutor:
         if self._executor is None:
             # assumes max_workers wont change
             self._executor = self._exit_stack.enter_context(
-                ThreadPoolExecutor(
+                InheritContextThreadPoolExecutor(
                     max_workers=max_workers,
                     thread_name_prefix="run_dequeue_worker",
                 )
@@ -69,21 +77,26 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         workspace_process_context: IWorkspaceProcessContext,
         fixed_iteration_time: Optional[float] = None,  # used for tests
     ) -> DaemonIterator:
-        run_coordinator = workspace_process_context.instance.run_coordinator
-        if not isinstance(run_coordinator, QueuedRunCoordinator):
-            check.failed(f"Expected QueuedRunCoordinator, got {run_coordinator}")
-
-        run_queue_config = run_coordinator.get_run_queue_config()
+        if not isinstance(workspace_process_context.instance.run_coordinator, QueuedRunCoordinator):
+            check.failed(
+                f"Expected QueuedRunCoordinator, got {workspace_process_context.instance.run_coordinator}"
+            )
+        run_coordinator = check.inst(
+            workspace_process_context.instance.run_coordinator, QueuedRunCoordinator
+        )
+        concurrency_config = workspace_process_context.instance.get_concurrency_config()
+        if not concurrency_config.run_queue_config:
+            check.failed("Got invalid run queue config")
 
         instance = workspace_process_context.instance
         runs_to_dequeue = self._get_runs_to_dequeue(
-            instance, run_queue_config, fixed_iteration_time=fixed_iteration_time
+            instance, concurrency_config, fixed_iteration_time=fixed_iteration_time
         )
         yield from self._dequeue_runs_iter(
             workspace_process_context,
             run_coordinator,
             runs_to_dequeue,
-            run_queue_config,
+            concurrency_config,
             fixed_iteration_time=fixed_iteration_time,
         )
 
@@ -91,8 +104,8 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         self,
         workspace_process_context: IWorkspaceProcessContext,
         run_coordinator: QueuedRunCoordinator,
-        runs_to_dequeue: List[DagsterRun],
-        run_queue_config: RunQueueConfig,
+        runs_to_dequeue: list[DagsterRun],
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
     ) -> Iterator[None]:
         if run_coordinator.dequeue_use_threads:
@@ -100,14 +113,14 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                 workspace_process_context,
                 runs_to_dequeue,
                 run_coordinator.dequeue_num_workers,
-                run_queue_config,
+                concurrency_config,
                 fixed_iteration_time=fixed_iteration_time,
             )
         else:
             yield from self._dequeue_runs_iter_loop(
                 workspace_process_context,
                 runs_to_dequeue,
-                run_queue_config,
+                concurrency_config,
                 fixed_iteration_time=fixed_iteration_time,
             )
 
@@ -115,23 +128,23 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         self,
         workspace_process_context: IWorkspaceProcessContext,
         run: DagsterRun,
-        run_queue_config: RunQueueConfig,
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
     ) -> bool:
         return self._dequeue_run(
             workspace_process_context.instance,
             workspace_process_context.create_request_context(),
             run,
-            run_queue_config,
+            concurrency_config,
             fixed_iteration_time,
         )
 
     def _dequeue_runs_iter_threaded(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-        runs_to_dequeue: List[DagsterRun],
+        runs_to_dequeue: list[DagsterRun],
         max_workers: Optional[int],
-        run_queue_config: RunQueueConfig,
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
     ) -> Iterator[None]:
         num_dequeued_runs = 0
@@ -141,7 +154,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                 self._dequeue_run_thread,
                 workspace_process_context,
                 run,
-                run_queue_config,
+                concurrency_config,
                 fixed_iteration_time=fixed_iteration_time,
             )
             for run in runs_to_dequeue
@@ -157,18 +170,17 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
     def _dequeue_runs_iter_loop(
         self,
         workspace_process_context: IWorkspaceProcessContext,
-        runs_to_dequeue: List[DagsterRun],
-        run_queue_config: RunQueueConfig,
+        runs_to_dequeue: list[DagsterRun],
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
     ) -> Iterator[None]:
         num_dequeued_runs = 0
-        workspace = workspace_process_context.create_request_context()
         for run in runs_to_dequeue:
             run_launched = self._dequeue_run(
                 workspace_process_context.instance,
-                workspace,
+                workspace_process_context.create_request_context(),
                 run,
-                run_queue_config,
+                concurrency_config,
                 fixed_iteration_time=fixed_iteration_time,
             )
             yield None
@@ -181,34 +193,33 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
     def _get_runs_to_dequeue(
         self,
         instance: DagsterInstance,
-        run_queue_config: RunQueueConfig,
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
-    ) -> List[DagsterRun]:
+    ) -> list[DagsterRun]:
         if not isinstance(instance.run_coordinator, QueuedRunCoordinator):
             check.failed(f"Expected QueuedRunCoordinator, got {instance.run_coordinator}")
 
+        run_queue_config = concurrency_config.run_queue_config
+        assert run_queue_config
         max_concurrent_runs = run_queue_config.max_concurrent_runs
         tag_concurrency_limits = run_queue_config.tag_concurrency_limits
 
-        in_progress_runs = self._get_in_progress_runs(instance)
+        in_progress_run_records = self._get_in_progress_run_records(instance)
+        in_progress_runs = [record.dagster_run for record in in_progress_run_records]
 
         max_concurrent_runs_enabled = max_concurrent_runs != -1  # setting to -1 disables the limit
-        max_runs_to_launch = max_concurrent_runs - len(in_progress_runs)
+        max_runs_to_launch = max_concurrent_runs - len(in_progress_run_records)
         if max_concurrent_runs_enabled:
             # Possibly under 0 if runs were launched without queuing
             if max_runs_to_launch <= 0:
                 self._logger.info(
-                    "{} runs are currently in progress. Maximum is {}, won't launch more.".format(
-                        len(in_progress_runs), max_concurrent_runs
-                    )
+                    f"{len(in_progress_run_records)} runs are currently in progress. Maximum is {max_concurrent_runs}, won't launch more."
                 )
                 return []
 
-        queued_runs = self._get_queued_runs(instance)
-
-        if not queued_runs:
-            self._logger.debug("Poll returned no queued runs.")
-            return []
+        cursor = None
+        has_more = True
+        batch: list[DagsterRun] = []
 
         now = fixed_iteration_time or time.time()
 
@@ -226,46 +237,113 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                 + ",".join(list(paused_location_names))
             )
 
-        self._logger.info(
-            f"Retrieved %d queued runs, checking limits.{locations_clause}", len(queued_runs)
-        )
+        logged_this_iteration = False
+        # Paginate through our runs list so we don't need to hold every run
+        # in memory at once. The maximum number of runs we'll hold in memory is
+        # max_runs_to_launch + page_size.
 
-        # place in order
-        sorted_runs = self._priority_sort(queued_runs)
-        tag_concurrency_limits_counter = TagConcurrencyLimitsCounter(
-            tag_concurrency_limits, in_progress_runs
-        )
+        concurrency_keys = None
+        pool_limits = None
 
-        batch: List[DagsterRun] = []
-        for run in sorted_runs:
-            if max_concurrent_runs_enabled and len(batch) >= max_runs_to_launch:
-                break
-
-            if tag_concurrency_limits_counter.is_blocked(run):
-                continue
-
-            location_name = (
-                run.external_pipeline_origin.location_name if run.external_pipeline_origin else None
+        while has_more:
+            queued_runs = instance.get_runs(
+                RunsFilter(statuses=[DagsterRunStatus.QUEUED]),
+                cursor=cursor,
+                limit=self._page_size,
+                ascending=True,
             )
-            if location_name and location_name in paused_location_names:
-                continue
+            has_more = len(queued_runs) >= self._page_size
 
-            tag_concurrency_limits_counter.update_counters_with_launched_item(run)
-            batch.append(run)
+            if not queued_runs:
+                has_more = False
+                return batch
+
+            if not logged_this_iteration:
+                logged_this_iteration = True
+                self._logger.info(
+                    "Priority sorting and checking tag concurrency limits for queued runs."
+                    + locations_clause
+                )
+
+            cursor = queued_runs[-1].run_id
+
+            tag_concurrency_limits_counter = TagConcurrencyLimitsCounter(
+                tag_concurrency_limits, in_progress_runs
+            )
+            batch += queued_runs
+            batch = self._priority_sort(batch)
+
+            if run_queue_config.should_block_op_concurrency_limited_runs:
+                try:
+                    # fetch global concurrency information at most once per iteration
+                    if concurrency_keys is None:
+                        concurrency_keys = instance.event_log_storage.get_concurrency_keys()
+
+                    if pool_limits is None:
+                        pool_limits = instance.event_log_storage.get_pool_limits()
+
+                    global_concurrency_limits_counter = GlobalOpConcurrencyLimitsCounter(
+                        instance,
+                        batch,
+                        in_progress_run_records,
+                        concurrency_keys=concurrency_keys,
+                        pool_limits=pool_limits,
+                        slot_count_offset=run_queue_config.op_concurrency_slot_buffer,
+                        pool_granularity=concurrency_config.pool_config.pool_granularity,
+                    )
+                except:
+                    self._logger.exception("Failed to initialize op concurrency counter")
+                    # when we cannot initialize the global concurrency counter, we should fall back
+                    # to not blocking any runs based on op concurrency limits
+                    global_concurrency_limits_counter = None
+            else:
+                global_concurrency_limits_counter = None
+
+            to_remove = []
+            for run in batch:
+                if tag_concurrency_limits_counter.is_blocked(run):
+                    to_remove.append(run)
+                    continue
+                else:
+                    tag_concurrency_limits_counter.update_counters_with_launched_item(run)
+
+                if (
+                    global_concurrency_limits_counter
+                    and global_concurrency_limits_counter.is_blocked(run)
+                ):
+                    to_remove.append(run)
+                    if run.run_id not in self._global_concurrency_blocked_runs:
+                        with self._global_concurrency_blocked_runs_lock:
+                            self._global_concurrency_blocked_runs.add(run.run_id)
+                        concurrency_blocked_info = json.dumps(
+                            global_concurrency_limits_counter.get_blocked_run_debug_info(run)
+                        )
+                        self._logger.info(
+                            f"Run {run.run_id} is blocked by global concurrency limits: {concurrency_blocked_info}"
+                        )
+                    continue
+                elif global_concurrency_limits_counter:
+                    global_concurrency_limits_counter.update_counters_with_launched_item(run)
+
+                location_name = (
+                    run.remote_job_origin.location_name if run.remote_job_origin else None
+                )
+                if location_name and location_name in paused_location_names:
+                    to_remove.append(run)
+                    continue
+
+            for run in to_remove:
+                batch.remove(run)
+
+            if max_runs_to_launch >= 1:
+                batch = batch[:max_runs_to_launch]
 
         return batch
 
-    def _get_queued_runs(self, instance: DagsterInstance) -> Sequence[DagsterRun]:
-        queued_runs_filter = RunsFilter(statuses=[DagsterRunStatus.QUEUED])
+    def _get_in_progress_run_records(self, instance: DagsterInstance) -> Sequence[RunRecord]:
+        return instance.get_run_records(filters=RunsFilter(statuses=IN_PROGRESS_RUN_STATUSES))
 
-        # Reversed for fifo ordering
-        runs = instance.get_runs(filters=queued_runs_filter)[::-1]
-        return runs
-
-    def _get_in_progress_runs(self, instance: DagsterInstance) -> Sequence[DagsterRun]:
-        return instance.get_runs(filters=RunsFilter(statuses=IN_PROGRESS_RUN_STATUSES))
-
-    def _priority_sort(self, runs: Iterable[DagsterRun]) -> Sequence[DagsterRun]:
+    def _priority_sort(self, runs: Iterable[DagsterRun]) -> list[DagsterRun]:
         def get_priority(run: DagsterRun) -> int:
             priority_tag_value = run.tags.get(PRIORITY_TAG, "0")
             try:
@@ -286,13 +364,17 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
     def _dequeue_run(
         self,
         instance: DagsterInstance,
-        workspace: IWorkspace,
+        workspace: BaseWorkspaceRequestContext,
         run: DagsterRun,
-        run_queue_config: RunQueueConfig,
+        concurrency_config: ConcurrencyConfig,
         fixed_iteration_time: Optional[float],
     ) -> bool:
+        assert concurrency_config.run_queue_config
         # double check that the run is still queued before dequeing
         run = check.not_none(instance.get_run_by_id(run.run_id))
+        with self._global_concurrency_blocked_runs_lock:
+            if run.run_id in self._global_concurrency_blocked_runs:
+                self._global_concurrency_blocked_runs.remove(run.run_id)
 
         now = fixed_iteration_time or time.time()
 
@@ -306,23 +388,19 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
 
         # Very old (pre 0.10.0) runs and programatically submitted runs may not have an
         # attached code location name
-        location_name = (
-            run.external_pipeline_origin.location_name if run.external_pipeline_origin else None
-        )
+        location_name = run.remote_job_origin.location_name if run.remote_job_origin else None
 
         if location_name and self._is_location_pausing_dequeues(location_name, now):
             self._logger.info(
-                (
-                    "Pausing dequeues for runs from code location %s to give its code server time"
-                    " to recover"
-                ),
+                "Pausing dequeues for runs from code location %s to give its code server time"
+                " to recover",
                 location_name,
             )
             return False
 
         launch_started_event = DagsterEvent(
             event_type_value=DagsterEventType.PIPELINE_STARTING.value,
-            pipeline_name=run.pipeline_name,
+            job_name=run.job_name,
         )
 
         instance.report_dagster_event(launch_started_event, run_id=run.run_id)
@@ -332,7 +410,11 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         try:
             instance.run_launcher.launch_run(LaunchRunContext(dagster_run=run, workspace=workspace))
         except Exception as e:
-            error = serializable_error_info_from_exc_info(sys.exc_info())
+            error = DaemonErrorCapture.process_exception(
+                exc_info=sys.exc_info(),
+                logger=self._logger,
+                log_message=f"Caught an error dequeuing run {run.run_id}",
+            )
 
             run = check.not_none(instance.get_run_by_id(run.run_id))
             # Make sure we don't re-enqueue a run if it has already finished or moved into STARTED:
@@ -342,14 +424,14 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                     f" {run.status} - moving on. Error: {error.to_string()}"
                 )
                 return False
-            elif run_queue_config.max_user_code_failure_retries and isinstance(
+            elif concurrency_config.run_queue_config.max_user_code_failure_retries and isinstance(
                 e, (DagsterUserCodeUnreachableError, DagsterCodeLocationLoadError)
             ):
                 if location_name:
                     with self._location_timeouts_lock:
                         # Don't try to dequeue runs from this location for another N seconds
                         self._location_timeouts[location_name] = (
-                            now + run_queue_config.user_code_failure_retry_delay
+                            now + concurrency_config.run_queue_config.user_code_failure_retry_delay
                         )
 
                 enqueue_event_records = instance.get_records_for_run(
@@ -360,13 +442,14 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
 
                 num_retries_so_far = len(enqueue_event_records) - 1
 
-                if num_retries_so_far >= run_queue_config.max_user_code_failure_retries:
+                if (
+                    num_retries_so_far
+                    >= concurrency_config.run_queue_config.max_user_code_failure_retries
+                ):
                     message = (
                         "Run dequeue failed to reach the user code server after"
-                        f" {run_queue_config.max_user_code_failure_retries} attempts, failing run"
+                        f" {concurrency_config.run_queue_config.max_user_code_failure_retries} attempts, failing run"
                     )
-                    message_with_full_error = f"{message}: {error.to_string()}"
-                    self._logger.error(message_with_full_error)
                     instance.report_engine_event(
                         message,
                         run,
@@ -376,26 +459,21 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                     return False
                 else:
                     retries_left = (
-                        run_queue_config.max_user_code_failure_retries - num_retries_so_far
+                        concurrency_config.run_queue_config.max_user_code_failure_retries
+                        - num_retries_so_far
                     )
                     retries_str = "retr" + ("y" if retries_left == 1 else "ies")
                     message = (
                         "Run dequeue failed to reach the user code server, re-submitting the run"
                         f" into the queue ({retries_left} {retries_str} remaining)"
                     )
-                    message_with_full_error = f"{message}: {error.to_string()}"
-                    self._logger.warning(message_with_full_error)
-
                     instance.report_engine_event(
                         message,
                         run,
                         EngineEventData.engine_error(error),
                     )
                     # Re-submit the run into the queue
-                    enqueued_event = DagsterEvent(
-                        event_type_value=DagsterEventType.PIPELINE_ENQUEUED.value,
-                        pipeline_name=run.pipeline_name,
-                    )
+                    enqueued_event = DagsterEvent.job_enqueue(run)
                     instance.report_dagster_event(enqueued_event, run_id=run.run_id)
                     return False
             else:
@@ -403,8 +481,6 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
                     "Caught an unrecoverable error while dequeuing the run. Marking the run as"
                     " failed and dropping it from the queue"
                 )
-                message_with_full_error = f"{message}: {error.to_string()}"
-                self._logger.error(message_with_full_error)
 
                 instance.report_engine_event(
                     message,
